from abc import abstractmethod, ABC
from enum import StrEnum
from typing import Optional
from uuid import UUID

from pydantic import BaseModel
from pydantic.dataclasses import dataclass


class SelectionMethod(StrEnum):
    LIST = 'list'
    ALL = 'all'
    RULES = 'rules'

class Operator(StrEnum):
    EQ = 'eq'
    NEQ = 'neq'
    GT = 'gt'
    GTE = 'gte'
    LT = 'lt'
    LTE = 'lte'
    IN = 'in'
    NIN = 'nin'
    REGEX = 'regex'
    LIKE = 'like'
    I_LIKE = 'iLike'
    EXISTS = 'exists'
    NOT_EXISTS = 'notExists'


class ResolvableModel(BaseModel, ABC):

    @staticmethod
    @abstractmethod
    async def resolve_object(object_id: UUID) -> "ResolvableModel":
        pass


class ResolvableCompanyModel(BaseModel, ABC):

    @staticmethod
    @abstractmethod
    async def resolve_object(object_id: UUID, organization_id: str) -> "ResolvableCompanyModel":
        pass


@dataclass
class DropdownOption:
    value: str
    label: str
    detail: str | None = None


@dataclass
class VacancySelectionRule:
    field: str
    operator: Operator
    value: Optional[str] = None


@dataclass
class VacancySelectionCriteria:
    method: SelectionMethod
    rules: Optional[list[VacancySelectionRule]] = None
    selected_ids: Optional[list[UUID]] = None


@dataclass
class FileData:
    file_name: str
    data: str
    content_type: str


from enum import StrEnum


class SalaryFrequency(StrEnum):
    MONTH = "MONTH"
    YEAR = "YEAR"
    WEEK = "WEEK"
    HOUR = "HOUR"

class RemoteType(StrEnum):
    ONSITE = "ONSITE"
    HYBRID = "HYBRID"
    REMOTE = "REMOTE"

class JobCategory(StrEnum):
    # Customer-facing & commercial
    CUSTOMER_SERVICE = "CUSTOMER_SERVICE"
    SALES = "SALES"
    BUSINESS_DEVELOPMENT = "BUSINESS_DEVELOPMENT"
    ACCOUNT_MANAGEMENT = "ACCOUNT_MANAGEMENT"

    # Marketing & communication
    MARKETING = "MARKETING"
    COMMUNICATIONS_PR = "COMMUNICATIONS_PR"
    CONTENT_COPY = "CONTENT_COPY"
    BRAND = "BRAND"
    EVENTS = "EVENTS"
    DIGITAL_MARKETING = "DIGITAL_MARKETING"

    # People & organization
    HR = "HR"
    RECRUITMENT = "RECRUITMENT"
    LEARNING_AND_DEVELOPMENT = "LEARNING_AND_DEVELOPMENT"

    # Finance & legal
    FINANCE_ACCOUNTING = "FINANCE_ACCOUNTING"
    CONTROLLING = "CONTROLLING"
    LEGAL = "LEGAL"
    COMPLIANCE_RISK = "COMPLIANCE_RISK"

    # Tech
    IT_SUPPORT = "IT_SUPPORT"
    SOFTWARE_ENGINEERING = "SOFTWARE_ENGINEERING"
    DATA_ANALYTICS = "DATA_ANALYTICS"
    DATA_SCIENCE_ML = "DATA_SCIENCE_ML"
    DEVOPS_SRE = "DEVOPS_SRE"
    SECURITY = "SECURITY"
    QA_TESTING = "QA_TESTING"
    PRODUCT_MANAGEMENT = "PRODUCT_MANAGEMENT"
    PROJECT_MANAGEMENT = "PROJECT_MANAGEMENT"
    UX_UI_DESIGN = "UX_UI_DESIGN"

    # Operations & supply chain
    OPERATIONS = "OPERATIONS"
    PROCUREMENT = "PROCUREMENT"
    LOGISTICS_SUPPLY_CHAIN = "LOGISTICS_SUPPLY_CHAIN"
    TRANSPORT = "TRANSPORT"
    WAREHOUSE = "WAREHOUSE"
    ADMINISTRATION = "ADMINISTRATION"
    OFFICE_MANAGEMENT = "OFFICE_MANAGEMENT"
    FACILITY_MANAGEMENT = "FACILITY_MANAGEMENT"

    # Skilled trades & industry
    ENGINEERING_TECHNICAL = "ENGINEERING_TECHNICAL"
    MAINTENANCE = "MAINTENANCE"
    MANUFACTURING_PRODUCTION = "MANUFACTURING_PRODUCTION"
    QUALITY_MANUFACTURING = "QUALITY_MANUFACTURING"
    CONSTRUCTION_INSTALLATION = "CONSTRUCTION_INSTALLATION"
    ENERGY_UTILITIES = "ENERGY_UTILITIES"

    # Sector-specific
    HEALTHCARE_MEDICAL = "HEALTHCARE_MEDICAL"
    EDUCATION_TRAINING = "EDUCATION_TRAINING"
    RESEARCH_SCIENCE = "RESEARCH_SCIENCE"
    GOVERNMENT_NONPROFIT = "GOVERNMENT_NONPROFIT"
    SECURITY_DEFENSE = "SECURITY_DEFENSE"

    # Retail & hospitality
    RETAIL = "RETAIL"
    HOSPITALITY = "HOSPITALITY"
    FOOD_SERVICE = "FOOD_SERVICE"

    # Creative & media
    DESIGN_CREATIVE = "DESIGN_CREATIVE"
    MEDIA_JOURNALISM = "MEDIA_JOURNALISM"

    # Consulting & advisory
    CONSULTING_ADVISORY = "CONSULTING_ADVISORY"

    OTHER = "OTHER"


class IndustryCategory(StrEnum):
    UNSPECIFIED = "UNSPECIFIED"

    ICT = "ICT"
    GOVERNMENT_NONPROFIT = "GOVERNMENT_NONPROFIT"
    FASHION_TEXTILES_COSMETICS = "FASHION_TEXTILES_COSMETICS"
    WHOLESALE_TRADE = "WHOLESALE_TRADE"
    BANKING_FINANCIAL_SERVICES = "BANKING_FINANCIAL_SERVICES"
    AUTOMOTIVE = "AUTOMOTIVE"
    INTERNET_SERVICES = "INTERNET_SERVICES"
    CONSULTING_ADVISORY = "CONSULTING_ADVISORY"
    RETAIL = "RETAIL"
    CONSTRUCTION_INSTALLATION = "CONSTRUCTION_INSTALLATION"
    TELECOMMUNICATIONS = "TELECOMMUNICATIONS"
    HOSPITALITY = "HOSPITALITY"
    CHEMICALS_PETROCHEMICALS = "CHEMICALS_PETROCHEMICALS"
    FACILITY_SERVICES = "FACILITY_SERVICES"
    INSURANCE = "INSURANCE"
    ENGINEERING_TECHNICAL = "ENGINEERING_TECHNICAL"
    EDUCATION_TRAINING = "EDUCATION_TRAINING"
    SPORTS_RECREATION_TOURISM = "SPORTS_RECREATION_TOURISM"
    INDUSTRIAL_MANUFACTURING = "INDUSTRIAL_MANUFACTURING"
    ELECTRONICS = "ELECTRONICS"
    FMCG = "FMCG"
    SECURITY_SERVICES = "SECURITY_SERVICES"
    BUSINESS_SERVICES = "BUSINESS_SERVICES"
    PASSENGER_TRANSPORT = "PASSENGER_TRANSPORT"
    STAFFING_RECRUITMENT = "STAFFING_RECRUITMENT"
    PHARMACEUTICALS = "PHARMACEUTICALS"
    AGRICULTURE_FORESTRY_FISHING = "AGRICULTURE_FORESTRY_FISHING"
    WASTE_ENVIRONMENT = "WASTE_ENVIRONMENT"
    MARITIME = "MARITIME"
    REAL_ESTATE = "REAL_ESTATE"
    LOGISTICS_TRANSPORT_DISTRIBUTION = "LOGISTICS_TRANSPORT_DISTRIBUTION"
    HEALTHCARE_WELLBEING = "HEALTHCARE_WELLBEING"
    LEGAL_SERVICES = "LEGAL_SERVICES"
    ACCOUNTING_AUDIT = "ACCOUNTING_AUDIT"
    ADVERTISING_PR_COMMUNICATIONS = "ADVERTISING_PR_COMMUNICATIONS"
    ARTS_CULTURE_ENTERTAINMENT = "ARTS_CULTURE_ENTERTAINMENT"
    MEDIA_PUBLISHING_BROADCASTING = "MEDIA_PUBLISHING_BROADCASTING"
    ENERGY_UTILITIES = "ENERGY_UTILITIES"

    OTHER = "OTHER"

class ExperienceLevel(StrEnum):
    UNSPECIFIED = "UNSPECIFIED"

    INTERN = "INTERN"
    ENTRY_LEVEL = "ENTRY_LEVEL"
    JUNIOR = "JUNIOR"
    MID_LEVEL = "MID_LEVEL"
    SENIOR = "SENIOR"
    LEAD = "LEAD"
    MANAGER = "MANAGER"
    SENIOR_MANAGER = "SENIOR_MANAGER"
    DIRECTOR = "DIRECTOR"
    EXECUTIVE = "EXECUTIVE"

    OTHER = "OTHER"

class Education(StrEnum):
    UNSPECIFIED = "UNSPECIFIED"

    # ISCED 2011 niveaus (UNESCO)
    NO_FORMAL_EDUCATION = "NO_FORMAL_EDUCATION"
    PRIMARY = "PRIMARY"
    LOWER_SECONDARY = "LOWER_SECONDARY"
    UPPER_SECONDARY = "UPPER_SECONDARY"
    POST_SECONDARY_NON_TERTIARY = "POST_SECONDARY_NON_TERTIARY"
    SHORT_CYCLE_TERTIARY = "SHORT_CYCLE_TERTIARY"
    BACHELOR = "BACHELOR"
    MASTER = "MASTER"
    DOCTORATE = "DOCTORATE"
    POSTDOCTORAL = "POSTDOCTORAL"

    OTHER = "OTHER"

class ContractType(StrEnum):
    UNSPECIFIED = "UNSPECIFIED"

    PERMANENT = "PERMANENT"              # indefinite / open-ended
    TEMPORARY = "TEMPORARY"              # fixed-term
    CONTRACTOR = "CONTRACTOR"            # contractor / 1099 / zelfstandige (zonder payroll)
    FREELANCE = "FREELANCE"              # freelancer (overlaps contractor; keep if channels distinguish)
    SELF_EMPLOYED = "SELF_EMPLOYED"      # zzp / independent (if you want explicit)
    INTERIM = "INTERIM"                  # interim assignment

    INTERN = "INTERN"                    # internship / stage
    APPRENTICESHIP = "APPRENTICESHIP"    # leer-werk / dual / BBL
    TEMP_AGENCY = "TEMP_AGENCY"          # via uitzendbureau / staffing
    VOLUNTEER = "VOLUNTEER"              # volunteer
    SEASONAL = "SEASONAL"                # seasonal work
    STUDENT_JOB = "STUDENT_JOB"          # part-time student job / bijbaan
    HOLIDAY_JOB = "HOLIDAY_JOB"          # vacation job

    FRANCHISE = "FRANCHISE"              # franchise / zelfstandige formule

    OTHER = "OTHER"


from enum import StrEnum


class ErrorCode(StrEnum):
    UNKNOWN_ERROR = "UNKNOWN_ERROR"
    NOT_AUTHENTICATED = "NOT_AUTHENTICATED"
    NOT_FOUND = "NOT_FOUND"
    BAD_REQUEST = "BAD_REQUEST"
    ARGUMENT_ERROR = "ARGUMENT_ERROR"
    NO_MEMBERSHIP = "NO_MEMBERSHIP"
    PERMISSION_DENIED = "PERMISSION_DENIED"
    ALREADY_EXISTS = "ALREADY_EXISTS"


class DisplayMessage(StrEnum):
    UNKNOWN_ERROR = "errors.unknownError"
    NOT_AUTHENTICATED = "errors.notAuthenticated"
    NOT_FOUND = "errors.notFound"
    BAD_REQUEST = "errors.badRequest"
    ARGUMENT_ERROR = "errors.argumentError"
    PERMISSION_DENIED = "errors.noPermission"
    NO_MEMBERSHIP = "errors.noMembership"
    ALREADY_EXISTS = "errors.alreadyExists"


class SKU(StrEnum):
    SYNCED_VACANCY = 'vacancy_sync'
    ENHANCE_VACANCY = 'vacancy_enhance'
    NEW_APPLICATION = 'application_new'




import os
from functools import wraps
from typing import Any, List

from aiocache import RedisCache
from aiocache.serializers import PickleSerializer

from ..util.singleton import SingletonMeta


class CacheService(metaclass=SingletonMeta):

    def __init__(self):
        endpoint = os.environ.get('CACHE_ENDPOINT', 'localhost')
        port = os.environ.get('CACHE_PORT', 6379)

        self._cache = RedisCache(endpoint=endpoint, port=port, serializer=PickleSerializer())

    @staticmethod
    def _get_namespace_and_key(key: str) -> tuple[str, str]:
        key_parts = key.split(":")
        return ':'.join(key_parts[1:]), key_parts[0]

    async def get(self, key: str) -> Any:
        key, namespace = self._get_namespace_and_key(key)

        if value := await self._cache.get(key, namespace=namespace):
            return value

        return None

    async def set(self, key, value, ttl=300):
        key, namespace = self._get_namespace_and_key(key)
        return await self._cache.set(key, value, ttl=ttl, namespace=namespace)

    async def delete(self, key):
        key, namespace = self._get_namespace_and_key(key)
        await self._cache.delete(key, namespace=namespace)


def cache_response(namespace: str, include_keys: List[str] = list, ttl: int = 300, debug=False):
    """
    Caching decorator for FastAPI endpoints.

    ttl: Time to live for the cache in seconds.
    namespace: Namespace for cache keys in Redis.
    """
    def decorator(func):

        @wraps(func)
        async def wrapper(*args, **kwargs):
            try:
                cache: CacheService = CacheService()
            except Exception as e:
                print(f"Error creating cache: {e}")
                return await func(*args, **kwargs)

            cache_key = f"{namespace}"

            # If organization is specific, add organization id to the cache key
            if organization_id := kwargs.get("x_organization_id"):
                cache_key += f":org:{organization_id}"

            for key in include_keys:
                value = kwargs.get(key)
                if value is not None:
                    cache_key += f":{key}:{value}"

            if debug:
                print(f"Cache key {cache_key}")

            # Try to retrieve data from cache
            try:
                if cached_value := await cache.get(cache_key):
                    return cached_value  # Return cached data
            except Exception as e:
                print(f"Error retrieving data from cache: {e}")

            response = await func(*args, **kwargs)

            try:
                # Store the response in Redis with a TTL
                await cache.set(cache_key, response, ttl=ttl)
            except Exception as e:
                print(f"Error caching data: {e}")

            return response

        return wrapper

    return decorator



import base64
import os
import tempfile
import traceback
from enum import StrEnum
from http import HTTPStatus

from fastapi import UploadFile
from google.api_core.exceptions import NotFound
from google.cloud import storage
from google.cloud.storage import Bucket, Blob
from talentro.constants import ErrorCode, DisplayMessage
from talentro.exceptions import APIException

from ..general.dataclasses import FileData
from ..util.singleton import SingletonMeta
from ..util.string import render_template_path


class BucketEnum(StrEnum):
    PUBLIC = "talentro-static"
    BILLING = "talentro-billing"
    CANDIDATES = "talentro-candidates"


class FilePathEnum(StrEnum):
    IAM_ORGANIZATION_LOGO = "iam/organizations/{organization_id}/details/logo.{extension}"
    IAM_PROFILE_PICTURE = "iam/users/{user_id}/profile/profile_picture.{extension}"
    CANDIDATES_CANDIDATE_CV_FILE = "organizations/{organization_id}/cand_{candidate_id}/cv/{file_name}"
    CANDIDATES_CANDIDATE_CV_FOLDER = "organizations/{organization_id}/cand_{candidate_id}/cv/"


class GoogleStorage(metaclass=SingletonMeta):

    def __init__(self):
        self._client = storage.Client()

    def upload_base64_to_gcs(self, file: FileData, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict) -> Blob:
        file_bytes = base64.b64decode(file.data)

        _, ext = os.path.splitext(file.file_name)
        tmp_fd, tmp_path = tempfile.mkstemp(suffix=ext)
        os.close(tmp_fd)

        blob = self._get_blob(bucket, destination, path_definitions)

        with open(tmp_path, "wb") as f:
            f.write(file_bytes)

        blob.upload_from_filename(
            tmp_path,
            content_type=file.content_type  # handig voor GCS metadata
        )

        if os.path.exists(tmp_path):
            os.remove(tmp_path)

        return blob

    def _get_bucket(self, bucket: BucketEnum) -> Bucket:
        try:
            bucket = self._client.get_bucket(bucket.value)
        except NotFound:
            print("Bucket not found.")
            bucket = self._client.create_bucket(bucket.value)

        return bucket

    def _get_blob(self, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict):
        bucket = self._get_bucket(bucket)

        try:
            filled_destination_path = render_template_path(destination, path_definitions)

        except ValueError as e:
            raise APIException(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                display_message=DisplayMessage.UNKNOWN_ERROR,
                message=str(e),
                error_code=ErrorCode.UNKNOWN_ERROR
            )

        blob = bucket.blob(filled_destination_path)
        return blob

    def get_blob(self, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict) -> Blob:
        """Deletes a blob from the bucket."""
        blob = self._get_blob(bucket, destination, path_definitions)
        blob.reload()
        return blob

    def delete_file(self, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict) -> bool:
        """Deletes a blob from the bucket."""
        try:
            blob = self._get_blob(bucket, destination, path_definitions)
            blob.reload()
            generation_match_precondition = blob.generation
            blob.delete(if_generation_match=generation_match_precondition)
            return True
        except NotFound as e:
            print(e)
            traceback.print_exc()
            return False

    def delete_folder(self, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict) -> bool:
        """Deletes a blob from the bucket."""
        try:
            bucket = self._get_bucket(bucket)
            filled_destination_path = render_template_path(destination, path_definitions)

            blobs = list(bucket.list_blobs(prefix=filled_destination_path))
            if blobs:
                bucket.delete_blobs(blobs)

            return True
        except NotFound as e:
            print(e)
            traceback.print_exc()
            return False

    def upload_file(self, file: UploadFile, bucket: BucketEnum, destination: FilePathEnum, path_definitions: dict) -> Blob:
        """
        Uploads a file to Google Storage

        :param file: The file you want to upload.
        :param bucket: The bucket you want to upload to.
        :param destination: The file destination you want to upload to.
        :param path_definitions: Key-value pair to declare the dynamic parts of the destination path.
        :return: The public URL of the uploaded file.
        :rtype: str
        """
        # Get the bucket or create it if it doesn't exist.
        blob = self._get_blob(bucket, destination, path_definitions)

        blob.upload_from_file(file.file)

        return blob



import os

from httpx import AsyncClient
from talentro.util.singleton import SingletonMeta


class MSClient(metaclass=SingletonMeta):

    _integrations_client: AsyncClient | None = None
    _acquisition_client: AsyncClient | None = None
    _vacancies_client: AsyncClient | None = None
    _candidates_client: AsyncClient | None = None

    @classmethod
    def integrations(cls) -> AsyncClient:
        if cls._integrations_client is None:
            cls._integrations_client = AsyncClient(
                base_url=f"{os.getenv('INTEGRATIONS_BASE_URL')}/api/v1/",
                timeout=30,
            )
        return cls._integrations_client

    @classmethod
    def acquisition(cls) -> AsyncClient:
        if cls._acquisition_client is None:
            cls._acquisition_client = AsyncClient(
                base_url=f"{os.getenv('ACQUISITION_BASE_URL')}/api/v1/",
                timeout=30,
            )
        return cls._acquisition_client

    @classmethod
    def vacancies(cls) -> AsyncClient:
        if cls._vacancies_client is None:
            cls._vacancies_client = AsyncClient(
                base_url=f"{os.getenv('VACANCIES_BASE_URL')}/api/v1/",
                timeout=30,
            )
        return cls._vacancies_client

    @classmethod
    def candidates(cls) -> AsyncClient:
        if cls._candidates_client is None:
            cls._candidates_client = AsyncClient(
                base_url=f"{os.getenv('CANDIDATES_BASE_URL')}/api/v1/",
                timeout=30,
            )
        return cls._candidates_client

    @classmethod
    async def aclose(cls) -> None:
        if cls._integrations_client is not None:
            await cls._integrations_client.aclose()
            cls._integrations_client = None
        if cls._acquisition_client is not None:
            await cls._acquisition_client.aclose()
            cls._acquisition_client = None
        if cls._vacancies_client is not None:
            await cls._vacancies_client.aclose()
            cls._vacancies_client = None
        if cls._candidates_client is not None:
            await cls._candidates_client.aclose()
            cls._candidates_client = None



import json
from hashlib import sha256

from talentro.vacancies.models import Vacancy as VacancyModel


def generate_vacancy_hash(vacancy_data: VacancyModel) -> str:
    data: dict = vacancy_data.model_dump()

    relevant_fields = {
        key: data[key]
        for key in [
            "title", "description", "status", "job_site_url", "company_name",
            "parent_company_name", "remote_type", "publish_date", "expiration_date",
            "last_updated_date", "category", "experience", "education", "hours_fte",
            "hours_min", "hours_max", "location_address", "location_zipcode",
            "location_city", "location_state", "location_country", "salary_min",
            "salary_max", "salary_currency", "salary_frequency", "recruiter_first_name",
            "recruiter_last_name", "recruiter_phone_number", "recruiter_email",
            "recruiter_role", "contract_type", "industry_category"
        ]
    }

    json_data = json.dumps(relevant_fields, sort_keys=True, default=str)
    return sha256(json_data.encode("utf-8")).hexdigest()


from datetime import datetime, timezone
from typing import Optional
from uuid import UUID

from sqlmodel import SQLModel, Field

from ..general.models import BaseModel


class BillingModel(BaseModel):
    pass


class BillingOrganizationModel(BillingModel):
    organization: UUID = Field(index=True)


class BillingEvent(SQLModel, table=True):
    id: int = Field(
        primary_key=True,
    )

    organization: UUID = Field(index=True)
    event_time: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc),
        nullable=False,
    )
    sku: str = Field(nullable=False)
    count: int = Field(nullable=False, default=1)
    sent_to_stripe: bool = Field(default=False)


class BillingProfile(BillingOrganizationModel, table=True):
    stripe_customer_id: str = Field(nullable=False)



from typing import Any


def safe_get(data: dict, attribute: str, default: Any = None, splitter:str = '.'):
    if attribute is None:
        return default

    for key in attribute.split(splitter):
        try:
            data = data[key]

            if data is None and default is not None:
                return default
        except KeyError:
            return default
        except TypeError:
            return default

    return data


from datetime import datetime
from typing import Optional
from uuid import UUID

from pydantic import BaseModel
from talentro.acquisition.dataclasses import CampaignInfo
from talentro.candidates.models import Application as ApplicationModel, Document as DocumentModel, \
    Candidate as CandidateModel
from talentro.vacancies.dataclasses import VacancyInfo


class CandidateInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    first_name: Optional[str] = None
    last_name: Optional[str] = None
    email: Optional[str] = None
    phone_number: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None

    @classmethod
    async def from_model(cls: "CandidateInfo", model: CandidateModel) -> 'CandidateInfo':
        return CandidateInfo(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,

            first_name=model.first_name,
            last_name=model.last_name,
            email=model.email,
            phone_number=model.phone_number,
            city=model.city,
            country=model.country,
        )


class ApplicationData(BaseModel):
    status: str
    source: str
    candidate_id: UUID
    vacancy_reference_number: str
    screening_answers: dict


class Application(ApplicationData):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID


class DocumentInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    type: str
    blob_name: str

    @classmethod
    async def from_model(cls: "DocumentInfo", model: DocumentModel) -> 'DocumentInfo':
        return DocumentInfo(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            type=model.type,
            blob_name=model.blob_name
        )


class ApplicationInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    status: str
    screening_answers: dict

    vacancy: Optional[VacancyInfo]
    campaign: Optional[CampaignInfo]
    candidate: Optional[CandidateInfo]

    documents: list[DocumentInfo]

    @classmethod
    async def from_model(cls: "ApplicationInfo", model: ApplicationModel) -> 'ApplicationInfo':
        if model.source:
            campaign = await CampaignInfo.resolve_object(model.source.campaign_id, model.organization)
        else:
            campaign = None

        if model.vacancy_id:
            vacancy = await VacancyInfo.resolve_object(model.vacancy_id, model.organization)
        else:
            vacancy = None

        if model.candidate:
            candidate = await CandidateInfo.from_model(model.candidate)
        else:
            candidate = None

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,

            screening_answers=model.screening_answers,
            status=model.status,

            vacancy=vacancy,
            campaign=campaign,
            candidate=candidate,

            documents=[await DocumentInfo.from_model(document) for document in model.documents],
        )



import string

def render_template_path(template: str, values: dict) -> str:
    formatter = string.Formatter()

    required_fields = [field for _, field, _, _ in formatter.parse(template) if field]

    missing = [field for field in required_fields if field not in values]

    if missing:
        raise ValueError(f"Variables not set: {', '.join(missing)}")

    return template.format(**values)